//
//  AbstractViewManager.m
//  DemoViewManager
//
//  Created by Douglas Sjoquist on 7/9/11.
//  Copyright 2011 Ivy Gulch, LLC. All rights reserved.
//

#import "AbstractViewManager.h"

@implementation AbstractViewManager

@synthesize title = title_;
@synthesize view = view_;

- (id) initWithNibName:(NSString *) nibNameOrNil bundle:nibBundleOrNil options:(NSDictionary *) options {
    if ((self = [super init])) {
        NSString *className = NSStringFromClass([self class]); 
        NSBundle *useBundle = nibBundleOrNil ? nibBundleOrNil : [NSBundle mainBundle];
        NSString *useNibName = nibNameOrNil ? nibNameOrNil : className;
        [useBundle loadNibNamed:useNibName owner:self options:options];
        
        // nib file should link the base view object, or it will crash
        if (!view_) {
            NSException *exception = 
            [NSException exceptionWithName:NSInternalInconsistencyException
                                    reason:[NSString stringWithFormat:@"-[%@ initWithNibName:bundle:options:] loaded the \"%@\" nib but the view outlet was not set.", className, useNibName]
                                  userInfo:nil];
            [exception raise];
        }
        
        [self viewDidLoad];
    }
    return self;
}

- (void) dealloc {
    [view_ release], view_ = nil;
    
    [super dealloc];
}

- (void)viewDidLoad {
    loaded_ = YES;
}

- (void)viewDidUnload {
    loaded_ = NO;
}

- (BOOL)isViewLoaded {
    return loaded_;
}

- (void) viewWillAppear:(BOOL)animated {
}

- (void) viewDidAppear:(BOOL)animated {
    visible_ = YES;
}

- (void) viewWillDisappear:(BOOL)animated {
}

- (void) viewDidDisappear:(BOOL)animated {
    visible_ = NO;
}

- (BOOL) isViewVisible {
    return visible_;
}

- (void) didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
}

- (void) didReceiveMemoryWarning {
    // do nothing by default
}

- (void) configureView {
}

@end
