//
//  DetailViewController.m
//  DemoViewManager
//
//  Created by Douglas Sjoquist on 7/9/11.
//  Copyright 2011 Ivy Gulch, LLC. All rights reserved.
//

#import "DetailViewController.h"
#import "RootViewController.h"

@implementation DetailViewController

@synthesize toolbar = toolbar_;
@synthesize contentView = contentView_;

@synthesize detailItem = detailItem_;
@synthesize popoverController = popoverController_;
@synthesize currentViewManager = currentViewManager_;
@synthesize webViewManager = webViewManager_;
@synthesize imageViewManager = imageViewManager_;

- (void)dealloc {
    [popoverController_ release];
    [toolbar_ release];
    [detailItem_ release];
    [currentViewManager_ release], currentViewManager_ = nil;
    [webViewManager_ release], webViewManager_ = nil;
    [imageViewManager_ release], imageViewManager_ = nil;
    
    [super dealloc];
}

- (void)viewDidUnload {
	[super viewDidUnload];
    
	self.popoverController = nil;
    self.currentViewManager = nil;
    self.imageViewManager = nil;
    self.webViewManager = nil;
}

- (void) didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    
    if (webViewManager_) {
        [webViewManager_ didReceiveMemoryWarning];
    }
    if (imageViewManager_) {
        [imageViewManager_ didReceiveMemoryWarning];
    }
}

- (void) setCurrentViewManager:(AbstractViewManager *)currentViewManager {
    if (currentViewManager == currentViewManager_) {
        return;
    }

    AbstractViewManager *oldViewManager = [currentViewManager_ autorelease];
    NSArray *subviews = [self.contentView subviews];
    
    if (oldViewManager) {
        [oldViewManager viewWillDisappear:YES];
    }
    for (id subview in subviews) {
        [subview removeFromSuperview];
    }
    if (oldViewManager) {
        [oldViewManager viewDidDisappear:YES];
    }
    
    currentViewManager.view.frame = self.contentView.bounds;
    if (currentViewManager) {
        [currentViewManager viewWillAppear:YES];
    }
    
    [self.contentView addSubview:currentViewManager.view];
    currentViewManager_ = [currentViewManager retain];    
    
    if (currentViewManager_) {
        [currentViewManager_ viewDidAppear:YES];
    }
}

- (WebViewManager *) webViewManager {
    if (webViewManager_ == nil) {
        webViewManager_ = [[WebViewManager alloc] initWithNibName:@"WebViewManager" bundle:nil options:nil];
        [webViewManager_ viewDidLoad];
    }
    return webViewManager_;
}

- (void) setWebViewManager:(WebViewManager *) webViewManager {
    if (webViewManager_ == webViewManager) {
        return;
    }
    
    [webViewManager_ viewDidUnload];
    [webViewManager_ autorelease];
    webViewManager_ = [webViewManager retain];
}

- (ImageViewManager *) imageViewManager {
    if (!imageViewManager_) {
        imageViewManager_ = [[ImageViewManager alloc] initWithNibName:@"ImageViewManager" bundle:nil options:nil];
        [imageViewManager_ viewDidLoad];
    }
    return imageViewManager_;
}

- (void) setImageViewManager:(ImageViewManager *) imageViewManager {
    if (imageViewManager_ == imageViewManager) {
        return;
    }
    
    [imageViewManager_ viewDidUnload];
    [imageViewManager_ autorelease];
    imageViewManager_ = [imageViewManager retain];
}

- (void)setDetailItem:(id)newDetailItem {
    [detailItem_ autorelease];
    detailItem_ = [newDetailItem retain];

    if (self.popoverController != nil) {
        [self.popoverController dismissPopoverAnimated:YES];
    }        

    NSString *detailType = [detailItem_ valueForKey:@"type"];
    if ([detailType isEqualToString:@"image"]) {
        self.currentViewManager = self.imageViewManager;
        [self.imageViewManager loadImage:[detailItem_ valueForKey:@"name"]
                                   title:[detailItem_ valueForKey:@"title"]];
    } else if ([detailType isEqualToString:@"web"]) {
        self.currentViewManager = self.webViewManager;
        [self.webViewManager loadURL:[detailItem_ valueForKey:@"url"]
                               title:[detailItem_ valueForKey:@"title"]];
    } else {
        self.currentViewManager = nil;
    }

}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    [self.currentViewManager viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    [self.currentViewManager viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
    
    [self.currentViewManager viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
    
    [self.currentViewManager viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

#pragma mark - Split view support

- (void)splitViewController:(UISplitViewController *)svc willHideViewController:(UIViewController *)aViewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController: (UIPopoverController *)pc {
    barButtonItem.title = @"Details";
    NSMutableArray *items = [[self.toolbar items] mutableCopy];
    [items insertObject:barButtonItem atIndex:0];
    [self.toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = pc;
}

- (void)splitViewController:(UISplitViewController *)svc willShowViewController:(UIViewController *)aViewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem {
    [self.toolbar setItems:nil animated:YES];
    self.popoverController = nil;
}

@end
